#include <QPainter>
#include "cgraphics2d.h"

CGraphics2D  graph2D;

//---------------------------------------------------------------------------------------------
CGraphics2D::CGraphics2D()
{
}

//---------------------------------------------------------------------------------------------
void    CGraphics2D::line(QImage* image, const CVec2& p1, const CVec2& p2, QColor color)
{
    if (image->valid(p1.x, p1.y) == false ||
        image->valid(p2.x, p2.y) == false)
        return;

    int32_t min, max;

    if (p1.y == p2.y)
    {
        // horizontal
        min = MIN(p1.x, p2.x);
        max = MAX(p1.x, p2.x);

        for (int32_t i = min; i <= max; ++i)
            image->setPixel(i, p1.y, color.rgba());
    }
    else
    {
        // vertical
        min = MIN(p1.y, p2.y);
        max = MAX(p1.y, p2.y);

        for (int32_t i = min; i <= max; ++i)
            image->setPixel(p1.x, i, color.rgba());
    }
}

//---------------------------------------------------------------------------------------------
void    CGraphics2D::rect(QImage* image, const CRect& r, QColor color, bool filled)
{
    if (image->valid(r.tl.x, r.tl.y) == false ||
        image->valid(r.br.x, r.br.y) == false)
        return;

    if (filled == false)
    {
        line(image, r.tl, CVec2(r.br.x, r.tl.y), color);
        line(image, CVec2(r.br.x, r.tl.y), r.br, color);
        line(image, r.tl, CVec2(r.tl.x, r.br.y), color);
        line(image, CVec2(r.tl.x, r.br.y), r.br, color);
    }
    else
    {
        int32_t min, max;

        min = MIN(r.tl.y, r.br.y);
        max = MAX(r.tl.y, r.br.y);

        for (int32_t i = min; i <= max; ++i)
            line(image, CVec2(r.tl.x, i), CVec2(r.br.x, i), color);
    }
}

//---------------------------------------------------------------------------------------------
void    CGraphics2D::drawImage(QImage* destImage, const CVec2& pos, const QImage& srcImage, int angle)
{
    QPainter painter(destImage);
    QPoint  pos2(pos.x, pos.y);

    if (angle == 0)
    {
        painter.drawImage(pos2, srcImage);
    }
    else
    {
        painter.translate(pos2);
        painter.translate((float)srcImage.width() / 2, (float)srcImage.height() / 2);
        painter.rotate(angle);
        painter.translate(-(float)srcImage.width() / 2, -(float)srcImage.height() / 2);
        painter.drawImage(0.5, 0.5, srcImage);
        painter.resetTransform();
    }

    painter.end();
}
